/**
* Loads in the vertices of a 3D mesh from the string
* contents of a ".OBJ" file.
*
* @param {string} The contents of an OBJ file.
*/
function loadMeshData(string) {
	var lines = string.split("\n");
	var positions = [];
	var normals = [];
	var uvs = [];
	var vertexBuffer = [];

	for(var i = 0; i < lines.length; i++) {
		var tokens = lines[i].trim().split(' ');
		if(tokens.length > 0){
			switch(tokens[0]){
				case 'v': positions.push([parseFloat(tokens[1]), parseFloat(tokens[2]), parseFloat(tokens[3])]); break;
				case 'vt': uvs.push([parseFloat(tokens[1]), parseFloat(tokens[2])]); break;
				case 'vn': normals.push([parseFloat(tokens[1]), parseFloat(tokens[2]), parseFloat(tokens[3])]); break;
				case 'f':
					for(var v = 1; v < 4; v++){
						var indexTokens = tokens[v].split("/");
						var vertexIndex = parseInt(indexTokens[0]) - 1;
						var uvIndex = parseInt(indexTokens[1]) - 1;
						var normalIndex = parseInt(indexTokens[2]) - 1;
						Array.prototype.push.apply(vertexBuffer, positions[vertexIndex])
						Array.prototype.push.apply(vertexBuffer, uvs[uvIndex])
						Array.prototype.push.apply(vertexBuffer, normals[normalIndex])
					}
				break;
			}
		}
	}
	var elementsPerVertex = 3 + 2 + 3;
	return {
		vertexBuffer: new Float32Array(vertexBuffer),
		vertexCount: vertexBuffer.length / elementsPerVertex,
	};
}
